#include "pixmap.h"
#include "window.h"
#include "colormap.h"
#include "bitmap.h"
 
void Gdk_Pixmap::ref()
  {
   if (obj_) 
     gdk_pixmap_ref(obj_);
  }

void Gdk_Pixmap::unref()
  {
   if (obj_) 
     gdk_pixmap_unref(obj_);
   obj_=0;
  }

Gdk_Pixmap::Gdk_Pixmap():Gdk_Drawable(0) 
  {
  }

Gdk_Pixmap::Gdk_Pixmap(GdkPixmap *pixmap):Gdk_Drawable(pixmap)
  {
   ref();
  }

Gdk_Pixmap::Gdk_Pixmap(Gdk_Pixmap const &pixmap):Gdk_Drawable(pixmap)
  {
   ref();
  }

Gdk_Pixmap::~Gdk_Pixmap()
  {
   free();
  }

/*****************************************************************/

void Gdk_Pixmap::create(const Gdk_Drawable  &window,
           gint        width,
           gint        height,
           gint        depth)
  {
   unref();
   obj_=gdk_pixmap_new(gdk_const_cast(window),width,height,depth);
   ref();
  }

Gdk_Pixmap::Gdk_Pixmap(const Gdk_Drawable &window,
           gint        width, 
           gint        height,
           gint        depth):Gdk_Drawable(0)
  {
   create(window,width,height,depth);
  }

void Gdk_Pixmap::create(
           gint        width,
           gint        height,
           gint        depth)
  {
   unref();
   obj_=gdk_pixmap_new(NULL,width,height,depth);
   ref();
  }

Gdk_Pixmap::Gdk_Pixmap(
           gint        width, 
           gint        height,
           gint        depth):Gdk_Drawable(0)
  {
   create(width,height,depth);
  }

/****************************************************************/

void Gdk_Pixmap::create_from_data(
           const Gdk_Drawable  &window,
           const gchar *data,
           gint        width,
           gint        height,
           gint        depth,
           const Gdk_Color   &fg,
           const Gdk_Color   &bg)
  {
   unref();
   obj_=gdk_pixmap_create_from_data(gdk_const_cast(window),
     const_cast<gchar*>(data),width,height,depth,
     gdk_const_cast(fg),
     gdk_const_cast(bg));  
   ref();
  }

Gdk_Pixmap::Gdk_Pixmap(const Gdk_Drawable  &window,
           const gchar      *data,
           gint        width,
           gint        height,
           gint        depth,
           const Gdk_Color  &fg,
           const Gdk_Color  &bg):Gdk_Drawable(0)
  {
   create_from_data(window,data,width,height,depth,fg,bg);
  }
    
void Gdk_Pixmap::create_from_xpm(
           const Gdk_Drawable  &window,
           const Gdk_Color   &transparent_color,
           const string      &filename)
  {
   unref();
   obj_=gdk_pixmap_create_from_xpm(gdk_const_cast(window),NULL,
     gdk_const_cast(transparent_color),
     const_cast<gchar*>(filename.c_str()));
   ref();
  }

void Gdk_Pixmap::create_from_xpm(
           const Gdk_Drawable  &window,
           Gdk_Bitmap  &mask,
           const Gdk_Color   &transparent_color,
           const string      &filename)
  {
   unref();
   GdkBitmap *bit;
   obj_=gdk_pixmap_create_from_xpm(gdk_const_cast(window),&bit,
     gdk_const_cast(transparent_color),
     const_cast<gchar*>(filename.c_str()));
   mask=Gdk_Bitmap(bit);
   ref();
  }
    
Gdk_Pixmap::Gdk_Pixmap(const Gdk_Drawable  &window,
           const Gdk_Color   &transparent_color,
           const string      &filename):Gdk_Drawable(0)
  {
   create_from_xpm(window,transparent_color,filename);
  }
 
Gdk_Pixmap::Gdk_Pixmap(const Gdk_Drawable  &window,
           Gdk_Bitmap  &bitmap,
           const Gdk_Color   &transparent_color,
           const string      &filename):Gdk_Drawable(0)
  {
   create_from_xpm(window,bitmap,transparent_color,filename);
  }  

void Gdk_Pixmap::create_colormap_from_xpm(
           const Gdk_Drawable   &window,
           Gdk_Colormap &colormap,
           Gdk_Bitmap   &bitmap,
           const Gdk_Color    &transparent_color,
           const string &filename)
  {
   unref();
   GdkBitmap *bit;
   obj_=gdk_pixmap_colormap_create_from_xpm(gdk_const_cast(window),
     colormap,&bit,
     gdk_const_cast(transparent_color),
     const_cast<gchar*>(filename.c_str()));
   ref();
   bitmap=Gdk_Bitmap(bit);
  }


Gdk_Pixmap::Gdk_Pixmap(const Gdk_Drawable   &window,
           Gdk_Colormap &colormap,
           Gdk_Bitmap   &bitmap,
           const Gdk_Color    &transparent_color,
           const string &filename):Gdk_Drawable(0)
  {
   create_colormap_from_xpm(window, colormap, bitmap,
                            transparent_color,filename);
  }
 
void Gdk_Pixmap::create_from_xpm_d(const Gdk_Drawable &window, 
                       Gdk_Bitmap       &bitmap,
                       const Gdk_Color  &transparent_color,
                       const gchar * const *data)
  {
   unref();
   GdkBitmap  *bit;
   obj_=gdk_pixmap_create_from_xpm_d(gdk_const_cast(window),&bit,
             gdk_const_cast(transparent_color),
             const_cast<gchar**>(data));
   ref();
   bitmap=Gdk_Bitmap(bit);
  }

void Gdk_Pixmap::create_colormap_from_xpm_d (const Gdk_Drawable   &window,
                                 Gdk_Colormap &colormap,
                                 Gdk_Bitmap  &mask,
                                 const Gdk_Color    &transparent_color,
                                 const gchar * const *data)
  {
   unref();
   GdkBitmap  *bit;
   obj_=gdk_pixmap_colormap_create_from_xpm_d(gdk_const_cast(window),
             colormap,
             &bit,gdk_const_cast(transparent_color),
             const_cast<gchar**>(data));
   ref();
   mask=Gdk_Bitmap(bit);
  }
